package lcsb.mapviewer.services.impl;

import java.awt.Color;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.annotation.PostConstruct;
import javax.mail.MessagingException;

import org.apache.commons.io.IOUtils;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import lcsb.mapviewer.annotation.services.MiriamConnector;
import lcsb.mapviewer.commands.ColorExtractor;
import lcsb.mapviewer.commands.ColorModelCommand;
import lcsb.mapviewer.commands.CommandExecutionException;
import lcsb.mapviewer.commands.CopyCommand;
import lcsb.mapviewer.common.IProgressUpdater;
import lcsb.mapviewer.common.Pair;
import lcsb.mapviewer.common.TextFileUtils;
import lcsb.mapviewer.common.exception.InvalidArgumentException;
import lcsb.mapviewer.common.exception.InvalidStateException;
import lcsb.mapviewer.converter.graphics.MapGenerator;
import lcsb.mapviewer.converter.graphics.MapGenerator.MapGeneratorParams;
import lcsb.mapviewer.model.Project;
import lcsb.mapviewer.model.cache.UploadedFileEntry;
import lcsb.mapviewer.model.log.LogType;
import lcsb.mapviewer.model.map.MiriamData;
import lcsb.mapviewer.model.map.MiriamType;
import lcsb.mapviewer.model.map.layout.ColorSchema;
import lcsb.mapviewer.model.map.layout.GeneVariation;
import lcsb.mapviewer.model.map.layout.GeneVariationColorSchema;
import lcsb.mapviewer.model.map.layout.GenericColorSchema;
import lcsb.mapviewer.model.map.layout.InvalidColorSchemaException;
import lcsb.mapviewer.model.map.layout.Layout;
import lcsb.mapviewer.model.map.layout.LayoutStatus;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.model.map.model.ModelSubmodelConnection;
import lcsb.mapviewer.model.map.reaction.Reaction;
import lcsb.mapviewer.model.map.species.Element;
import lcsb.mapviewer.model.user.ObjectPrivilege;
import lcsb.mapviewer.model.user.PrivilegeType;
import lcsb.mapviewer.model.user.User;
import lcsb.mapviewer.persist.DbUtils;
import lcsb.mapviewer.persist.dao.map.LayoutDao;
import lcsb.mapviewer.services.SecurityException;
import lcsb.mapviewer.services.interfaces.IConfigurationService;
import lcsb.mapviewer.services.interfaces.ILayoutService;
import lcsb.mapviewer.services.interfaces.ILogService;
import lcsb.mapviewer.services.interfaces.ILogService.LogParams;
import lcsb.mapviewer.services.interfaces.IUserService;
import lcsb.mapviewer.services.search.layout.FullLayoutAliasView;
import lcsb.mapviewer.services.search.layout.FullLayoutAliasViewFactory;
import lcsb.mapviewer.services.search.layout.FullLayoutReactionView;
import lcsb.mapviewer.services.search.layout.FullLayoutReactionViewFactory;
import lcsb.mapviewer.services.search.layout.LightLayoutAliasView;
import lcsb.mapviewer.services.search.layout.LightLayoutAliasViewFactory;
import lcsb.mapviewer.services.search.layout.LightLayoutReactionView;
import lcsb.mapviewer.services.search.layout.LightLayoutReactionViewFactory;
import lcsb.mapviewer.services.utils.ColorSchemaReader;
import lcsb.mapviewer.services.utils.EmailSender;
import lcsb.mapviewer.services.utils.data.ColorSchemaColumn;
import lcsb.mapviewer.services.view.AuthenticationToken;
import lcsb.mapviewer.services.view.LayoutView;
import lcsb.mapviewer.services.view.LayoutViewFactory;

/**
 * Implementation of the layout service.
 * 
 * @author Piotr Gawron
 * 
 */
@Transactional(value = "txManager")
public class LayoutService implements ILayoutService {

  /**
   * Default class logger.
   */
  private Logger logger = Logger.getLogger(LayoutService.class);

  /**
   * Layout data access object.
   */
  @Autowired
  private LayoutDao layoutDao;

  /**
   * Factory object used for creation of {@link LayoutView} elements.
   */
  @Autowired
  private LayoutViewFactory layoutViewFactory;

  /**
   * Service that manages and gives access to user information.
   */
  @Autowired
  private IUserService userService;

  /**
   * Service used to access logs.
   *
   * @see ILogService
   */
  @Autowired
  private ILogService logService;

  /**
   * Service that manages and gives access to configuration parameters.
   */
  @Autowired
  private IConfigurationService configurationService;

  /**
   * Utils that help to manage the sessions in custom multithreaded
   * implementation.
   */
  @Autowired
  private DbUtils dbUtils;

  /**
   * Object that sends emails.
   */
  private EmailSender emailSender;

  /**
   * Method called after spring initialized all interfaces.
   */
  @PostConstruct
  public void springInit() {
    emailSender = new EmailSender(configurationService);
  }

  @Override
  public boolean userCanAddLayout(Model model, User user) {
    // if we don't have privileges to view the object then we cannot add layouts
    if (!userService.userHasPrivilege(user, PrivilegeType.VIEW_PROJECT, model.getProject())) {
      return false;
    }
    long count = getAvailableCustomLayoutsNumber(user);
    return count > 0;
  }

  @Override
  public long getAvailableCustomLayoutsNumber(User user) {
    long level = userService.getUserPrivilegeLevel(user, PrivilegeType.CUSTOM_LAYOUTS);
    long layouts = layoutDao.getCountByUser(user);
    return level - layouts;

  }

  @Override
  public boolean userCanRemoveLayout(LayoutView layout, User user) {
    Layout originalLayout = layoutDao.getById(layout.getIdObject());
    User creator = originalLayout.getCreator();
    Project project = originalLayout.getModel().getProject();
    if (creator == null) {
      return userService.userHasPrivilege(user, PrivilegeType.LAYOUT_MANAGEMENT, project);
    } else {
      return creator.getId().equals(user.getId())
          || userService.userHasPrivilege(user, PrivilegeType.LAYOUT_MANAGEMENT, project);
    }
  }

  @Override
  public List<LayoutView> getCustomLayouts(Model model, User user, Boolean publicOverlay, User creator) {
    // TODO remove publicOverlay and creator params
    List<LayoutView> result = new ArrayList<>();
    if (model == null || user == null) {
      return result;
    }
    List<Layout> layouts = layoutDao.getLayoutsByModel(model, user, false);
    for (Layout layout : layouts) {
      result.add(layoutViewFactory.create(layout));
    }
    Collections.sort(result, LayoutView.ID_COMPARATOR);
    return result;
  }

  private boolean userCanViewOverlay(Layout overlay, User user) {
    if (overlay.isPublicLayout()) {
      return true;
    }
    if (overlay.getCreator() == null) {
      return true;
    }
    if (overlay.getCreator().getLogin().equals(user.getLogin())) {
      return true;
    }

    if (userService.userHasPrivilege(user, PrivilegeType.LAYOUT_VIEW, overlay)) {
      return true;
    }
    return false;
  }

  @Override
  public List<LayoutView> getGeneralLayouts(Model model) {
    List<LayoutView> result = new ArrayList<LayoutView>();
    if (model == null) {
      return result;
    }
    List<Layout> layouts = layoutDao.getLayoutsByModel(model);
    for (Layout layout : layouts) {
      if (layout.isPublicLayout()) {
        result.add(layoutViewFactory.create(layout));
      }
    }
    Collections.sort(result, LayoutView.ID_COMPARATOR);
    return result;
  }

  @Override
  public void removeLayout(LayoutView layout, final String homeDir) throws IOException {
    final String dir;
    final Layout dbLayout = layoutDao.getById(layout.getIdObject());
    if (homeDir != null) {
      if (dbLayout.getModel().getProject().getDirectory() != null) {
        dir = homeDir + "/../map_images/" + dbLayout.getModel().getProject().getDirectory() + "/";
      } else {
        dir = homeDir + "/../map_images/";
      }
    } else {
      dir = null;
    }

    String projectId = dbLayout.getModel().getProject().getProjectId();
    String email = null;
    User user = dbLayout.getCreator();
    if (user != null) {
      email = user.getEmail();
    }

    dbLayout.getModel().removeLayout(dbLayout);
    layoutDao.delete(dbLayout);

    LogParams params = new LogParams().object(dbLayout).type(LogType.LAYOUT_REMOVED);
    logService.log(params);

    if (email != null) {
      try {
        sendSuccesfullRemoveEmail(projectId, layout.getName(), email);
      } catch (MessagingException e) {
        logger.error(e);
      }
    }
    Thread computations = new Thread(new Runnable() {
      @Override
      public void run() {
        MapGenerator generator = new MapGenerator();
        try {
          generator.removeLayout(dbLayout, dir);
        } catch (IOException e) {
          logger.error(e);
        }

      }
    });
    computations.start();
  }

  @Override
  public void updateLayout(LayoutView layout) {
    Layout dbLayout = layoutDao.getById(layout.getIdObject());
    dbLayout.setTitle(layout.getName());
    dbLayout.setDescription(layout.getDescription());
    if (layout.getCreator().isEmpty()) {
      dbLayout.setCreator(null);
      dbLayout.setPublicLayout(true);
    } else {
      User user = userService.getUserByLogin(layout.getCreator());
      dbLayout.setCreator(user);
      dbLayout.setPublicLayout(false);
    }

    layoutDao.update(dbLayout);
  }

  @Override
  public void addViewPrivilegeToLayout(LayoutView layout, User user) {
    ObjectPrivilege privilege = new ObjectPrivilege();
    privilege.setIdObject(layout.getIdObject());
    privilege.setLevel(1);
    privilege.setType(PrivilegeType.LAYOUT_VIEW);
    privilege.setUser(user);
    userService.setUserPrivilege(user, privilege);
  }

  @Override
  public void dropViewPrivilegeFromLayout(LayoutView layout, User user) {
    ObjectPrivilege privilege = new ObjectPrivilege();
    privilege.setIdObject(layout.getIdObject());
    privilege.setLevel(0);
    privilege.setType(PrivilegeType.LAYOUT_VIEW);
    privilege.setUser(user);
    userService.setUserPrivilege(user, privilege);
  }

  @Override
  public LayoutView createLayoutWithImages(final CreateLayoutParams params)
      throws IOException, InvalidColorSchemaException, CommandExecutionException {
    ColorSchemaReader reader = new ColorSchemaReader();
    final Collection<ColorSchema> schemas = reader.readColorSchema(params.getColorInputStream(),
        TextFileUtils.getHeaderParametersFromFile(params.getColorInputStream()));
    final Model colorModel = new CopyCommand(params.getModel()).execute();
    new ColorModelCommand(params.getModel(), schemas, userService.getColorExtractorForUser(params.getUser())).execute();
    String[] tmp = params.getDirectory().split("[\\\\/]");
    String simpleDir = tmp[tmp.length - 1];

    layoutDao.flush();

    boolean customSession = false;
    if (params.isAsync()) {
      customSession = !dbUtils.isCustomSessionForCurrentThread();
      if (customSession) {
        dbUtils.createSessionForCurrentThread();
      }
    }
    final Map<Model, Integer> layoutIdByModel = new HashMap<>();

    Layout topLayout = new Layout(params.getName(), simpleDir, false);
    if (params.getUser() == null) {
      topLayout.setPublicLayout(true);
    } else {
      topLayout.setPublicLayout(false);
    }
    topLayout.setStatus(LayoutStatus.NA);
    topLayout.setProgress(0.0);
    UploadedFileEntry fileEntry = new UploadedFileEntry();
    fileEntry.setFileContent(IOUtils.toByteArray(params.getColorInputStream()));
    fileEntry.setOriginalFileName(params.getLayoutFileName());
    fileEntry.setLength(fileEntry.getFileContent().length);
    fileEntry.setOwner(params.getUser());
    topLayout.setInputData(fileEntry);
    topLayout.setDescription(params.getDescription());
    params.getModel().addLayout(topLayout);
    topLayout.setCreator(params.getUser());
    layoutDao.add(topLayout);
    topLayout.setDirectory(simpleDir + topLayout.getId());

    layoutDao.update(topLayout);

    layoutIdByModel.put(colorModel, topLayout.getId());

    for (ModelSubmodelConnection connection : params.getModel().getSubmodelConnections()) {
      Layout layout = new Layout(params.getName(), simpleDir, false);
      layout.setStatus(LayoutStatus.NA);
      layout.setProgress(0.0);
      connection.getSubmodel().getModel().addLayout(layout);
      layout.setCreator(params.getUser());
      layout.setPublicLayout(false);
      topLayout.addLayout(layout);
      layoutDao.add(layout);
      layout.setDirectory(simpleDir + layout.getId());

      layoutDao.update(layout);

      layoutIdByModel.put(colorModel.getSubmodelByConnectionName(connection.getName()), layout.getId());
    }

    if (params.isAsync()) {
      if (customSession) {
        dbUtils.closeSessionForCurrentThread();
      } else {
        layoutDao.commit();
      }
    }
    final int layoutId = layoutIdByModel.get(colorModel);

    Thread computations = new Thread(new Runnable() {
      @Override
      public void run() {
        if (params.isAsync()) {
          // open transaction for this thread
          dbUtils.createSessionForCurrentThread();
        }

        try {
          MapGenerator generator = new MapGenerator();
          final int models = layoutIdByModel.entrySet().size();

          int count = 0;
          for (Model m : layoutIdByModel.keySet()) {
            final int counted = count;
            int id = layoutIdByModel.get(m);
            MapGeneratorParams imgParams = generator.new MapGeneratorParams().model(m)
                .directory(params.getDirectory() + id).updater(new IProgressUpdater() {
                  private int lastProgress = -1;

                  @Override
                  public void setProgress(double progress) {
                    progress = progress / ((double) models) + ((double) counted * MAX_PROGRESS) / ((double) models);
                    if (((int) progress) != lastProgress) {
                      Layout layout = layoutDao.getById(layoutId);
                      lastProgress = (int) progress;
                      layout.setProgress(progress);
                      layout.setStatus(LayoutStatus.GENERATING);
                      layoutDao.update(layout);
                      if (params.isAsync()) {
                        layoutDao.commit();
                      }
                    }
                  }
                });
            imgParams.sbgn(params.getModel().getProject().isSbgnFormat());
            generator.generateMapImages(imgParams);
            count++;
          }
          Layout layout = layoutDao.getById(layoutId);
          layout.setProgress(IProgressUpdater.MAX_PROGRESS);
          layout.setStatus(LayoutStatus.OK);
          layoutDao.update(layout);
          if (params.isAsync()) {
            layoutDao.commit();
          }

          try {
            sendSuccessfullGenerationEmail(params, schemas);
          } catch (MessagingException e) {
            logger.error("Problem with sending email", e);
          }
          logService
              .log(new LogParams().object(layout).description("Created successfully.").type(LogType.LAYOUT_CREATED));
        } catch (Exception e) {
          logger.error("Problem with creating layout", e);

          Layout layout = layoutDao.getById(layoutId);
          layout.setProgress(IProgressUpdater.MAX_PROGRESS);
          layout.setStatus(LayoutStatus.FAILURE);
          layoutDao.update(layout);

          try {
            emailSender.sendEmail("MapViewer status", "There was a problem with generating layout " + params.getName()
                + ". Contact administrator to solve this issue.", params.getUser());
          } catch (MessagingException e1) {
            logger.error("Problem with sending email", e1);
          }
          logService
              .log(new LogParams().object(layout).description("Problem during creation.").type(LogType.LAYOUT_CREATED));
        }
        if (params.isAsync()) {
          // close the transaction for this thread
          dbUtils.closeSessionForCurrentThread();
        }
      }
    });

    if (params.isAsync()) {
      computations.start();
    } else {
      computations.run();
    }
    return layoutViewFactory.create(topLayout);
  }

  @Override
  public LayoutView createLayout(final CreateLayoutParams params) throws IOException, InvalidColorSchemaException {
    ColorSchemaReader reader = new ColorSchemaReader();
    final Collection<ColorSchema> schemas = reader.readColorSchema(params.getColorInputStream(),
        TextFileUtils.getHeaderParametersFromFile(params.getColorInputStream()));

    // check if we can color our model using this schema,
    // if not then exception will be thrown and passed up
    try {
      Model copy = new CopyCommand(params.getModel()).execute();
      new ColorModelCommand(copy, schemas, userService.getColorExtractorForUser(params.getUser())).execute();
    } catch (CommandExecutionException e) {
      throw new InvalidColorSchemaException(e);
    }

    String[] tmp = params.getDirectory().split("[\\\\/]");
    String simpleDir = tmp[tmp.length - 1];

    layoutDao.flush();

    boolean customSession = false;

    if (params.isAsync()) {
      customSession = !dbUtils.isCustomSessionForCurrentThread();
      if (customSession) {
        dbUtils.createSessionForCurrentThread();
      }
    }

    Layout topLayout = new Layout(params.getName(), simpleDir, false);
    if (params.getUser() == null) {
      topLayout.setPublicLayout(true);
    } else {
      topLayout.setPublicLayout(false);
    }
    topLayout.setStatus(LayoutStatus.OK);
    topLayout.setProgress(0.0);
    UploadedFileEntry fileEntry = new UploadedFileEntry();
    fileEntry.setFileContent(IOUtils.toByteArray(params.getColorInputStream()));
    fileEntry.setOriginalFileName(params.getLayoutFileName());
    fileEntry.setLength(fileEntry.getFileContent().length);
    fileEntry.setOwner(params.getUser());
    topLayout.setInputData(fileEntry);
    topLayout.setDescription(params.getDescription());
    params.getModel().addLayout(topLayout);
    topLayout.setCreator(params.getUser());
    layoutDao.add(topLayout);
    topLayout.setDirectory(simpleDir + topLayout.getId());

    layoutDao.update(topLayout);

    for (ModelSubmodelConnection connection : params.getModel().getSubmodelConnections()) {
      Layout layout = new Layout(params.getName(), simpleDir, false);
      layout.setStatus(LayoutStatus.OK);
      layout.setProgress(0.0);
      connection.getSubmodel().getModel().addLayout(layout);
      layout.setCreator(params.getUser());
      layout.setPublicLayout(false);
      topLayout.addLayout(layout);
      layoutDao.add(layout);
      layout.setDirectory(simpleDir + layout.getId());

      layoutDao.update(layout);
    }

    if (params.isAsync()) {
      if (customSession) {
        dbUtils.closeSessionForCurrentThread();
      } else {
        layoutDao.commit();
      }
    }
    Thread computations = new Thread(new Runnable() {
      @Override
      public void run() {
        if (params.isAsync()) {
          // open transaction for this thread
          dbUtils.createSessionForCurrentThread();
        }

        try {
          sendSuccessfullGenerationEmail(params, schemas);
        } catch (MessagingException e) {
          logger.error("Problem with sending email", e);
        }

        if (params.isAsync()) {
          // close the transaction for this thread
          dbUtils.closeSessionForCurrentThread();
        }
      }
    });

    LogParams logParams = new LogParams().object(topLayout).description("Created successfully.")
        .type(LogType.LAYOUT_CREATED);
    logService.log(logParams);

    if (params.isAsync()) {
      computations.start();
    } else {
      computations.run();
    }
    return layoutViewFactory.create(topLayout);
  }

  /**
   * @return the configurationService
   * @see #configurationService
   */
  public IConfigurationService getConfigurationService() {
    return configurationService;
  }

  /**
   * @param configurationService
   *          the configurationService to set
   * @see #configurationService
   */
  public void setConfigurationService(IConfigurationService configurationService) {
    this.configurationService = configurationService;
  }

  /**
   * @return the dbUtils
   * @see #dbUtils
   */
  public DbUtils getDbUtils() {
    return dbUtils;
  }

  /**
   * @param dbUtils
   *          the dbUtils to set
   * @see #dbUtils
   */
  public void setDbUtils(DbUtils dbUtils) {
    this.dbUtils = dbUtils;
  }

  /**
   * @return the userService
   * @see #userService
   */
  public IUserService getUserService() {
    return userService;
  }

  /**
   * @param userService
   *          the userService to set
   * @see #userService
   */
  public void setUserService(IUserService userService) {
    this.userService = userService;
  }

  @Override
  public LayoutView getLayoutByName(Model model, String name) {
    if (model == null) {
      return null;
    }
    Layout layout = layoutDao.getLayoutByName(model, name);
    if (layout != null) {
      return layoutViewFactory.create(layout);
    } else {
      return null;
    }
  }

  /**
   * Sends notification email that layout was removed.
   *
   * @param projectId
   *          identifier of the project
   * @param layoutName
   *          name of the layout
   * @param email
   *          otification email
   * @throws MessagingException
   *           thrown when there is a problem with sending email
   */
  protected void sendSuccesfullRemoveEmail(String projectId, String layoutName, String email)
      throws MessagingException {
    StringBuilder content = new StringBuilder(
        "Layout " + layoutName + " in map " + projectId + " was successfully removed.<br/>");
    emailSender.sendEmail("MapViewer notification", content.toString(), email);
  }

  /**
   * Sends notification email that layout was generated.
   *
   * @param params
   *          list of {@link CreateLayoutParams params} used for layout creation
   * @param schemas
   *          set of schemas used in coloring
   * @throws MessagingException
   *           thrown when there is a problem with sending email
   */
  protected void sendSuccessfullGenerationEmail(final CreateLayoutParams params, final Collection<ColorSchema> schemas)
      throws MessagingException {
    StringBuilder content = new StringBuilder("Layout " + params.getName() + " generated successfully.\n");
    content.append("Result of coloring:<br/>");

    String table = prepareTableResult(schemas, new ColorSchemaReader());
    content.append(table);
    String email = params.getModel().getProject().getNotifyEmail();
    if (params.getUser() != null) { // send email to a user who owns the layout
      emailSender.sendEmail("MapViewer notification", content.toString(), params.getUser());

      // send email to the model owner
      if (email != null && !email.equals(params.getUser().getEmail())) {
        content = new StringBuilder("");
        String username = "UNKNOWN";
        if (params.getUser() != null) {
          username = params.getUser().getLogin();
        }
        content.append(
            "User " + username + " created layout in " + params.getModel().getProject().getProjectId() + " map.");

        emailSender.sendEmail("MapViewer notification", content.toString(), email);
      }
    } else if (email != null) {
      // if nobody owns the layout then send it to the model owner
      emailSender.sendEmail("MapViewer notification", content.toString(), email);
    }
  }

  /**
   * Prepares table with statistics about coloring.
   *
   * @param schemas
   *          schemas that were used for coloring
   * @param scr
   *          interface that returns list of columns that should be printed
   * @return table with statistics about coloring
   */
  protected String prepareTableResult(Collection<ColorSchema> schemas, ColorSchemaReader scr) {
    StringBuilder sb = new StringBuilder("");

    Collection<ColorSchemaColumn> columns = scr.getSetColorSchemaColumns(schemas);

    for (ColorSchemaColumn column : ColorSchemaColumn.values()) {
      if (columns.contains(column)) {
        sb.append(column.getTitle() + "\t");
      }
    }
    sb.append("matches<br/>\n");

    for (ColorSchema originalSchema : schemas) {
      for (ColorSchema schema : splitColorSchema(originalSchema)) {
        sb.append(prepareTableRow(columns, schema));
      }
    }

    return sb.toString();
  }

  /**
   * {@link ColorSchema} sometimes contains merged value from few rows. This
   * method split single {@link ColorSchema} object to simple flat objects that
   * can be serialiazed in a simple tab separated file.
   *
   * @param originalSchema
   *          original {@link ColorSchema} objcet that we want to spli into flat
   *          objects
   * @return {@link List} of flat schema objects obtained from input
   */
  private List<ColorSchema> splitColorSchema(ColorSchema originalSchema) {
    List<ColorSchema> result = new ArrayList<>();
    if (originalSchema instanceof GenericColorSchema) {
      result.add(originalSchema);
    } else if (originalSchema instanceof GeneVariationColorSchema) {
      for (GeneVariation gv : ((GeneVariationColorSchema) originalSchema).getGeneVariations()) {
        GeneVariationColorSchema copy = (GeneVariationColorSchema) originalSchema.copy();
        copy.getGeneVariations().clear();
        copy.addGeneVariation(gv.copy());
        result.add(copy);
      }
    } else {
      throw new InvalidArgumentException("Unknown class type: " + originalSchema.getClass());
    }
    return result;
  }

  /**
   * Prepares tab separated {@link String} represenation of {@link ColorSchema}.
   *
   * @param columns
   *          columns that should be outputed in the result String
   * @param schema
   *          input ColorSchema that is going to be transformed into String
   *          representation
   * @return tab separated {@link String} represenation of {@link ColorSchema}
   */
  protected String prepareTableRow(Collection<ColorSchemaColumn> columns, ColorSchema schema) {
    StringBuilder sb = new StringBuilder();
    for (ColorSchemaColumn column : ColorSchemaColumn.values()) {
      if (columns.contains(column)) {
        if (schema instanceof GenericColorSchema) {
          sb.append(prepareTableCellForGenericSchema((GenericColorSchema) schema, column));
        } else if (schema instanceof GeneVariationColorSchema) {
          sb.append(prepareTableCellForGeneVariationSchema((GeneVariationColorSchema) schema, column));
        } else {
          throw new InvalidArgumentException("Unknown schema type: " + schema.getClass());
        }
      }
    }
    sb.append(schema.getMatches() + "<br/>\n");
    return sb.toString();
  }

  /**
   * Returns String representing data of {@link GenericColorSchema} that should
   * appear in a given {@link ColorSchemaColumn}.
   *
   * @param schema
   *          object for which data will be returned
   * @param column
   *          column for which data should be returned
   * @return {@link String} representing data of {@link GenericColorSchema} that
   *         should appear in a given {@link ColorSchemaColumn}
   */
  protected String prepareTableCellForGenericSchema(GenericColorSchema schema, ColorSchemaColumn column) {
    StringBuilder sb = new StringBuilder();
    if (column.equals(ColorSchemaColumn.COLOR)) {
      if (schema.getColor() != null) {
        sb.append("#" + Integer.toHexString(schema.getColor().getRGB()).substring(2).toUpperCase());
      }
      sb.append("\t");
    } else if (column.equals(ColorSchemaColumn.NAME)) {
      sb.append(schema.getName() + "\t");
    } else if (column.equals(ColorSchemaColumn.MODEL_NAME)) {
      sb.append(schema.getModelName() + "\t");
    } else if (column.equals(ColorSchemaColumn.VALUE)) {
      sb.append(schema.getValue() + "\t");
    } else if (column.equals(ColorSchemaColumn.COMPARTMENT)) {
      for (String str : schema.getCompartments()) {
        sb.append(str + ", ");
      }
      sb.append("\t");
    } else if (column.equals(ColorSchemaColumn.TYPE)) {
      for (Class<? extends Element> str : schema.getTypes()) {
        sb.append(str.getSimpleName() + ", ");
      }
      sb.append("\t");
    } else if (column.equals(ColorSchemaColumn.IDENTIFIER)) {
      for (Pair<MiriamType, String> pair : schema.getIdentifierColumns()) {
        if (pair.getRight() != null && !pair.getRight().trim().equals("")) {
          sb.append(pair.getLeft().getCommonName() + ": " + pair.getRight() + ", ");
        }
      }
      sb.append("\t");
    } else if (column.equals(ColorSchemaColumn.ELEMENT_IDENTIFIER)) {
      sb.append(schema.getElementId() + "\t");
    } else if (column.equals(ColorSchemaColumn.LINE_WIDTH)) {
      sb.append(schema.getLineWidth() + "\t");
    } else if (column.equals(ColorSchemaColumn.REVERSE_REACTION)) {
      sb.append(schema.getReverseReaction() + "\t");
    } else if (column.equals(ColorSchemaColumn.POSITION)) {
      sb.append(schema.getReverseReaction() + "\t");
    } else if (column.equals(ColorSchemaColumn.DESCRIPTION)) {
      sb.append(schema.getDescription() + "\t");
    } else {
      throw new InvalidArgumentException("Unknown column type: " + column + " for schema type: " + schema.getClass());
    }
    return sb.toString();
  }

  /**
   * Returns String representing data of {@link GeneVariationColorSchema} that
   * should appear in a given {@link ColorSchemaColumn}.
   *
   * @param schema
   *          object for which data will be returned
   * @param column
   *          column for which data should be returned
   * @return {@link String} representing data of {@link GeneVariationColorSchema}
   *         that should appear in a given {@link ColorSchemaColumn}
   */
  protected String prepareTableCellForGeneVariationSchema(GeneVariationColorSchema schema, ColorSchemaColumn column) {
    StringBuilder sb = new StringBuilder();
    if (column.equals(ColorSchemaColumn.COLOR)) {
      if (schema.getColor() != null) {
        sb.append("#" + Integer.toHexString(schema.getColor().getRGB()).substring(2).toUpperCase());
      }
      sb.append("\t");
    } else if (column.equals(ColorSchemaColumn.NAME)) {
      sb.append(schema.getName() + "\t");
    } else if (column.equals(ColorSchemaColumn.MODEL_NAME)) {
      sb.append(schema.getModelName() + "\t");
    } else if (column.equals(ColorSchemaColumn.VALUE)) {
      sb.append(schema.getValue() + "\t");
    } else if (column.equals(ColorSchemaColumn.COMPARTMENT)) {
      for (String str : schema.getCompartments()) {
        sb.append(str + ", ");
      }
      sb.append("\t");
    } else if (column.equals(ColorSchemaColumn.TYPE)) {
      for (Class<? extends Element> str : schema.getTypes()) {
        sb.append(str.getSimpleName() + ", ");
      }
      sb.append("\t");
    } else if (column.equals(ColorSchemaColumn.IDENTIFIER)) {
      for (Pair<MiriamType, String> pair : schema.getIdentifierColumns()) {
        if (pair.getRight() != null && !pair.getRight().trim().equals("")) {
          sb.append(pair.getLeft().getCommonName() + ": " + pair.getRight() + ", ");
        }
      }
      sb.append("\t");
    } else if (column.equals(ColorSchemaColumn.ELEMENT_IDENTIFIER)) {
      sb.append(schema.getElementId() + "\t");
    } else if (column.equals(ColorSchemaColumn.LINE_WIDTH)) {
      sb.append(schema.getLineWidth() + "\t");
    } else if (column.equals(ColorSchemaColumn.REVERSE_REACTION)) {
      sb.append(schema.getReverseReaction() + "\t");
    } else if (column.equals(ColorSchemaColumn.POSITION)) {
      sb.append(schema.getGeneVariations().get(0).getPosition() + "\t");
    } else if (column.equals(ColorSchemaColumn.DESCRIPTION)) {
      sb.append(schema.getDescription() + "\t");
    } else if (column.equals(ColorSchemaColumn.ORIGINAL_DNA)) {
      sb.append(schema.getGeneVariations().get(0).getOriginalDna() + "\t");
    } else if (column.equals(ColorSchemaColumn.ALTERNATIVE_DNA)) {
      sb.append(schema.getGeneVariations().get(0).getModifiedDna() + "\t");
    } else if (column.equals(ColorSchemaColumn.REFERENCE_GENOME_TYPE)) {
      sb.append(schema.getGeneVariations().get(0).getReferenceGenomeType() + "\t");
    } else if (column.equals(ColorSchemaColumn.REFERENCE_GENOME_VERSION)) {
      sb.append(schema.getGeneVariations().get(0).getReferenceGenomeVersion() + "\t");
    } else if (column.equals(ColorSchemaColumn.CONTIG)) {
      sb.append(schema.getGeneVariations().get(0).getContig() + "\t");
    } else if (column.equals(ColorSchemaColumn.REFERENCES)) {
      MiriamConnector mc = new MiriamConnector();
      for (MiriamData md : schema.getGeneVariations().get(0).getReferences()) {
        sb.append(mc.miriamDataToUri(md) + "\t");
      }
    } else {
      throw new InvalidArgumentException("Unknown column type: " + column + " for schema type: " + schema.getClass());
    }
    return sb.toString();
  }

  @Override
  public byte[] getInputDataForLayout(LayoutView layoutView, AuthenticationToken token) throws SecurityException {
    return getInputDataForLayout(layoutView.getIdObject(), token);
  }

  /**
   * Returns byte array containing data from original input file that was used to
   * generate the layout.
   *
   * @param layoutId
   *          identifier of layout for which we want to retrieve original file
   *          data
   * @return original data file for given layout, if such file is not stored in
   *         database (compatibility reasons) then null is returned
   * @throws SecurityException
   */
  private byte[] getInputDataForLayout(int layoutId, AuthenticationToken token) throws SecurityException {
    Layout layout = getLayoutDataById(layoutId, token);
    if (layout == null) {
      return null;
    } else {
      if (layout.getInputData() != null) {
        return layout.getInputData().getFileContent();
      } else {
        return null;
      }
    }
  }

  @Override
  public List<LightLayoutAliasView> getAliasesForLayout(Model model, int layoutId, AuthenticationToken token)
      throws SecurityException {
    try {
      ColorSchemaReader reader = new ColorSchemaReader();
      Collection<ColorSchema> schemas = reader.readColorSchema(getInputDataForLayout(layoutId, token));
      // colors here are not important
      ColorModelCommand command = new ColorModelCommand(model, schemas,
          new ColorExtractor(Color.BLACK, Color.BLACK, Color.BLACK));
      LightLayoutAliasViewFactory factory = new LightLayoutAliasViewFactory();
      List<LightLayoutAliasView> result = new ArrayList<>();
      for (Map.Entry<Object, ColorSchema> entry : command.getModifiedElements().entrySet()) {
        if (entry.getKey() instanceof Element) {
          result.add(factory.create(new Pair<Element, ColorSchema>((Element) entry.getKey(), entry.getValue())));
        }
      }
      return result;
    } catch (InvalidColorSchemaException e) {
      throw new InvalidStateException(e);
    } catch (IOException e) {
      throw new InvalidStateException(e);
    }
  }

  @Override
  public List<LightLayoutReactionView> getReactionsForLayout(Model model, int layoutId, AuthenticationToken token)
      throws SecurityException {
    try {
      ColorSchemaReader reader = new ColorSchemaReader();
      Collection<ColorSchema> schemas = reader.readColorSchema(getInputDataForLayout(layoutId, token));
      // colors here are not important
      ColorModelCommand command = new ColorModelCommand(model, schemas,
          new ColorExtractor(Color.BLACK, Color.BLACK, Color.BLACK));
      LightLayoutReactionViewFactory factory = new LightLayoutReactionViewFactory();
      List<LightLayoutReactionView> result = new ArrayList<>();
      for (Map.Entry<Object, ColorSchema> entry : command.getModifiedElements().entrySet()) {
        if (entry.getKey() instanceof Reaction) {
          result.add(factory.create(new Pair<Reaction, ColorSchema>((Reaction) entry.getKey(), entry.getValue())));
        }
      }
      return result;
    } catch (InvalidColorSchemaException e) {
      throw new InvalidStateException(e);
    } catch (IOException e) {
      throw new InvalidStateException(e);
    }
  }

  @Override
  public Map<Object, ColorSchema> getElementsForLayout(Model model, Integer layoutId, AuthenticationToken token)
      throws SecurityException {
    try {
      ColorSchemaReader reader = new ColorSchemaReader();
      Collection<ColorSchema> schemas;
      schemas = reader.readColorSchema(getInputDataForLayout(layoutId, token));
      // colors here are not important
      ColorModelCommand command = new ColorModelCommand(model, schemas,
          new ColorExtractor(Color.BLACK, Color.BLACK, Color.BLACK));
      return command.getModifiedElements();
    } catch (InvalidColorSchemaException e) {
      throw new InvalidStateException(e);
    } catch (IOException e) {
      throw new InvalidStateException(e);
    }
  }

  @Override
  public List<FullLayoutAliasView> getFullAliasesForLayoutByIds(Model model, List<Pair<Integer, Integer>> identifiers,
      int layoutId, AuthenticationToken token) throws SecurityException {
    try {
      Set<Integer> ids = new HashSet<>();
      for (Pair<Integer, Integer> pair : identifiers) {
        ids.add(pair.getRight());
      }

      ColorSchemaReader reader = new ColorSchemaReader();
      Collection<ColorSchema> schemas;
      schemas = reader.readColorSchema(getInputDataForLayout(layoutId, token));
      // colors here are not important
      ColorModelCommand command = new ColorModelCommand(model, schemas,
          new ColorExtractor(Color.BLACK, Color.BLACK, Color.BLACK));
      FullLayoutAliasViewFactory factory = new FullLayoutAliasViewFactory();
      List<FullLayoutAliasView> result = new ArrayList<>();

      // TODO poor performance, improve it
      for (Map.Entry<Object, ColorSchema> entry : command.getModifiedElements().entrySet()) {
        if (entry.getKey() instanceof Element) {
          Element alias = (Element) entry.getKey();
          if (ids.contains(alias.getId())) {
            result.add(factory.create(new Pair<Element, ColorSchema>(alias, entry.getValue())));
          }
        }
      }
      return result;
    } catch (InvalidColorSchemaException e) {
      throw new InvalidStateException(e);
    } catch (IOException e) {
      throw new InvalidStateException(e);
    }
  }

  @Override
  public EmailSender getEmailSender() {
    return emailSender;
  }

  @Override
  public void setEmailSender(EmailSender emailSender) {
    this.emailSender = emailSender;
  }

  @Override
  public List<LayoutView> getCustomLayouts(Model model, String token, Boolean publicOverlay, User creator)
      throws SecurityException {
    return this.getCustomLayouts(model, userService.getUserByToken(token), publicOverlay, creator);
  }

  @Override
  public LayoutView getLayoutById(int overlayId, AuthenticationToken token) throws SecurityException {
    Layout layout = getLayoutDataById(overlayId, token);
    if (layout == null) {
      return null;
    }
    return layoutViewFactory.create(layout);
  }

  @Override
  public Layout getLayoutDataById(int overlayId, AuthenticationToken token) throws SecurityException {
    Layout layout = layoutDao.getById(overlayId);
    if (layout == null) {
      return null;
    }
    User user = userService.getUserByToken(token);
    if (!userCanViewOverlay(layout, user)) {
      throw new SecurityException("User doesn't have access to overlay");
    }
    return layout;
  }

  @Override
  public FullLayoutAliasView getFullAliasForLayout(Model model, Integer id, int layoutId, AuthenticationToken token)
      throws SecurityException {
    try {
      ColorSchemaReader reader = new ColorSchemaReader();
      Collection<ColorSchema> schemas;
      schemas = reader.readColorSchema(getInputDataForLayout(layoutId, token));
      // colors here are not important
      ColorModelCommand command = new ColorModelCommand(model, schemas,
          new ColorExtractor(Color.BLACK, Color.BLACK, Color.BLACK));
      FullLayoutAliasViewFactory factory = new FullLayoutAliasViewFactory();

      for (Map.Entry<Object, ColorSchema> entry : command.getModifiedElements().entrySet()) {
        if (entry.getKey() instanceof Element) {
          Element alias = (Element) entry.getKey();
          if (id.equals(alias.getId())) {
            return factory.create(new Pair<Element, ColorSchema>(alias, entry.getValue()));
          }
        }
      }
      return null;
    } catch (InvalidColorSchemaException e) {
      throw new InvalidStateException(e);
    } catch (IOException e) {
      throw new InvalidStateException(e);
    }
  }

  @Override
  public FullLayoutReactionView getFullReactionForLayout(Model model, Integer id, int layoutId,
      AuthenticationToken token) throws SecurityException {
    try {
      ColorSchemaReader reader = new ColorSchemaReader();
      Collection<ColorSchema> schemas;
      schemas = reader.readColorSchema(getInputDataForLayout(layoutId, token));
      // colors here are not important
      ColorModelCommand command = new ColorModelCommand(model, schemas,
          new ColorExtractor(Color.BLACK, Color.BLACK, Color.BLACK));
      FullLayoutReactionViewFactory factory = new FullLayoutReactionViewFactory();

      for (Map.Entry<Object, ColorSchema> entry : command.getModifiedElements().entrySet()) {
        if (entry.getKey() instanceof Reaction) {
          Reaction alias = (Reaction) entry.getKey();
          if (id.equals(alias.getId())) {
            return factory.create(new Pair<Reaction, ColorSchema>(alias, entry.getValue()));
          }
        }
      }
      return null;
    } catch (InvalidColorSchemaException e) {
      throw new InvalidStateException(e);
    } catch (IOException e) {
      throw new InvalidStateException(e);
    }
  }

  @Override
  public boolean userCanRemoveLayout(LayoutView layout, AuthenticationToken authenticationToken) {
    User user = userService.getUserByToken(authenticationToken);
    return userCanRemoveLayout(layout, user);
  }
}
