package lcsb.mapviewer.services.search.data;

import java.util.ArrayList;
import java.util.Collections;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;

import lcsb.mapviewer.annotation.services.MiriamConnector;
import lcsb.mapviewer.common.exception.InvalidArgumentException;
import lcsb.mapviewer.common.exception.InvalidClassException;
import lcsb.mapviewer.common.exception.NotImplementedException;
import lcsb.mapviewer.model.map.MiriamData;
import lcsb.mapviewer.model.map.MiriamType;
import lcsb.mapviewer.model.map.reaction.Modifier;
import lcsb.mapviewer.model.map.reaction.Product;
import lcsb.mapviewer.model.map.reaction.Reactant;
import lcsb.mapviewer.model.map.reaction.Reaction;
import lcsb.mapviewer.model.map.reaction.ReactionNode;
import lcsb.mapviewer.services.search.ElementViewFactory;
import lcsb.mapviewer.services.view.AnnotationViewFactory;

/**
 * Factory class used to create {@link FullReactionView} elements from
 * {@link Reaction} objects.
 * 
 * @author Piotr Gawron
 *
 */
public class FullReactionViewFactory extends ElementViewFactory<Reaction, FullReactionView> {

	/**
	 * Default class logger.
	 */
	private static Logger					logger = Logger.getLogger(FullReactionViewFactory.class);

	/**
	 * Factory used to create {@link lcsb.mapviewer.services.view.AnnotationView
	 * AnnotationView} elements.
	 */
	@Autowired
	private AnnotationViewFactory	annotationViewFactory;

	/**
	 * Factory used to create {@link FullAliasView} elements.
	 */
	@Autowired
	private FullAliasViewFactory	fullAliasViewFactory;

	/**
	 * Connector to miriam database.
	 */
	@Autowired
	private MiriamConnector				mc;

	@Override
	public FullReactionView create(Reaction reaction) {
		FullReactionView result = new FullReactionView(reaction);
		result.setGeneProteinReaction(convertToHtml(reaction.getGeneProteinReaction()));

		ArrayList<MiriamData> miriamList = new ArrayList<MiriamData>(reaction.getMiriamData());
		Collections.sort(miriamList);
		result.setReferences(annotationViewFactory.createList(reaction.getMiriamData()));

		for (ReactionNode node : reaction.getReactionNodes()) {
			FullAliasView fav = fullAliasViewFactory.create(node.getElement());
			if (node instanceof Reactant) {
				fav.addOther("NodeType", "Reactant");
			} else if (node instanceof Product) {
				fav.addOther("NodeType", "Product");
			} else if (node instanceof Modifier) {
				fav.addOther("NodeType", "Modifier");
			} else {
				throw new InvalidClassException("Unknown reaction node type: " + node.getClass());
			}
			result.getElements().add(fav);
		}

		return result;
	}

	@Override
	public String createGson(FullReactionView object) {
		throw new NotImplementedException();
	}

	/**
	 * Converts
	 * {@link lcsb.mapviewer.model.map.reaction.Reaction#geneProteinReaction} into
	 * html tags.
	 * 
	 * @param string
	 *          string to convert
	 * @return html string with hyperlinks to external databases
	 */
	public String convertToHtml(String string) {
		if (string == null) {
			return null;
		}
		String result = "";
		int index = 0;

		while (index >= 0) {
			int begin = string.indexOf("(", index);
			int end = -1;
			if (begin >= 0) {
				end = string.indexOf(")", begin);
			}
			if (end > 0) {
				result += string.substring(index, begin);
				String identifier = string.substring(begin + 1, end);
				try {
					MiriamData md = MiriamType.getMiriamByUri(identifier);
					String link = mc.getUrlString(md);
					if (link != null) {
						result += "<a target=\"_blank\" href=\"" + link + "\"> " + md.getDataType().getCommonName() + " (" + md.getResource() + ")</a>";
					} else {
						logger.warn("Cannot find url for: " + string);
					}
				} catch (InvalidArgumentException e) {
					logger.warn("Unknown miriam urn: " + identifier);
					result += "(" + identifier + ")";
				}
				index = end + 1;
			} else {
				if (begin >= 0) {
					logger.warn("Invalid GeneProteinReaction string: " + string);
				}
				index = -1;
			}
		}
		return result;
	}

}
