package lcsb.mapviewer.services.view;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;

import com.google.gson.Gson;

import lcsb.mapviewer.annotation.data.Article;
import lcsb.mapviewer.annotation.services.MiriamConnector;
import lcsb.mapviewer.annotation.services.PubmedParser;
import lcsb.mapviewer.annotation.services.PubmedSearchException;
import lcsb.mapviewer.model.map.MiriamData;
import lcsb.mapviewer.model.map.MiriamType;
import lcsb.mapviewer.model.map.graph.DataMiningType;

/**
 * Factory class for {@link AnnotationView} class.
 * 
 * @author Piotr Gawron
 * 
 */
public class AnnotationViewFactory extends AbstractViewFactory<MiriamData, AnnotationView> {
	/**
	 * Default class logger.
	 */
	private static Logger		logger = Logger.getLogger(AnnotationViewFactory.class);

	/**
	 * Local backend to the pubmed data.
	 */
	@Autowired
	private PubmedParser		pubmedParser;

	/**
	 * Local backend to the miriam registry.
	 */
	@Autowired
	private MiriamConnector	miriamConnector;

	@Override
	public AnnotationView create(MiriamData object) {
		AnnotationView result = new AnnotationView(object);
		if (object != null && object.getDataType() != null) {
			result.setResource(object.getResource());
			if (object.getDataType().getUris().size() > 0) {
				try {
					result.setLink(miriamConnector.getUrlString(object));
				} catch (Exception e) {
					logger.error("Problem with miriam: " + object, e);
				}
			}
			if (MiriamType.PUBMED.equals(object.getDataType())) {
				try {
					String summary = pubmedParser.getSummary(object.getResource());
					// workaround,
					// tak a look at Bug #461
					if (summary != null) {
						result.setSummary(summary.replace("\\", "\\\\"));
					}
				} catch (PubmedSearchException e) {
					logger.error("Problem with accessing info about pubmed", e);
					result.setSummary("N/A");
				}
			}
			result.setType(object.getDataType().getCommonName());
			result.setName(object.getResource());
		}
		return result;
	}

	/**
	 * Creates {@link AnnotationView} for {@link DataMiningType} object.
	 * 
	 * @param dataMiningType
	 *          type of datamining element
	 * @return {@link AnnotationView} for {@link DataMiningType} object
	 */
	public AnnotationView create(DataMiningType dataMiningType) {
		AnnotationView result = new AnnotationView(null);
		result.setName(dataMiningType.getCommonName());
		if (dataMiningType.getSource() != null) {
			result.setLink(DataMiningType.STRUCTURE_ANALYSIS.getSource());
		}
		return result;
	}

	@Override
	public String createGson(AnnotationView object) {
		return new Gson().toJson(object);
	}

	@Override
	public MiriamData viewToObject(AnnotationView view) {
		MiriamType type = MiriamType.getTypeByCommonName(view.getType());
		return new MiriamData(type, view.getName());
	}

	/**
	 * Creates {@link AnnotationView} from an {@link Article}.
	 * 
	 * @param article
	 *          object from which we create {@link AnnotationView}
	 * @return {@link AnnotationView} from an {@link Article}
	 */
	public AnnotationView create(Article article) {
		return create(new MiriamData(MiriamType.PUBMED, article.getId()));
	}

}
