package lcsb.mapviewer.services.view;

import java.awt.geom.Point2D;

import org.apache.log4j.Logger;

import com.google.gson.Gson;

import lcsb.mapviewer.common.Configuration;
import lcsb.mapviewer.common.exception.InvalidArgumentException;
import lcsb.mapviewer.common.exception.NotImplementedException;
import lcsb.mapviewer.model.map.Comment;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.model.map.reaction.Reaction;
import lcsb.mapviewer.model.map.species.Element;

/**
 * Factory class for {@link CommentView} class.
 * 
 * @author Piotr Gawron
 * 
 */
public class CommentViewFactory extends AbstractViewFactory<Comment, CommentView> {
	/**
	 * Default class logger.
	 */
	private static Logger logger = Logger.getLogger(CommentViewFactory.class);

	@Override
	public CommentView create(Comment comment) {
		CommentView result = new CommentView(comment);
		if (comment == null) {
			return result;
		}
		Model model = comment.getModelData().getModel();
		return create(comment, model);
	}

	/**
	 * Creates a {@link CommentView} object for given comment.
	 * 
	 * @param comment
	 *          comment for which view is created
	 * @param model
	 *          model where comments is placed
	 * @return {@link CommentView} object for given comment
	 */
	public CommentView create(Comment comment, Model model) {
		CommentView result = new CommentView(comment);
		if (comment == null) {
			return result;
		}

		result.setAuthor(comment.getName());
		result.setContent(comment.getContent());
		result.setEmail(comment.getEmail());
		result.setPinned(comment.isPinned());
		if (comment.isDeleted()) {
			result.setRemoved("YES (" + comment.getRemoveReason() + ")");
		} else {
			result.setRemoved("NO");
		}
		if (comment.getSubmodelData() != null) {
			result.setSubmap(comment.getSubmodelData().getId() + "");
			model = model.getSubmodelById(comment.getSubmodelData().getId());
			if (model == null) {
				throw new InvalidArgumentException("Cannot find submodel with id: " + comment.getSubmodelData().getId());
			}
		}

		String title = "";
		Point2D coordinates = null;
		if (comment.getCoordinates() != null) {
			title = "Comment (coord: " + String.format("%.2f", comment.getCoordinates().getX()) + ", " + String.format("%.2f", comment.getCoordinates().getY())
					+ ")";
			coordinates = comment.getCoordinates();
		}
		if (comment.getTableName() != null) {
			if (comment.getTableName().getName().contains("Reaction")) {
				Reaction reaction = model.getReactionByDbId(comment.getTableId());
				if (reaction != null) {
					title = "Reaction " + reaction.getIdReaction();
					coordinates = reaction.getCenterPoint();
				} else {
					logger.warn("Invalid reaction dbID: " + comment.getTableId());
				}

			} else {
				Element alias = model.getElementByDbId(comment.getTableId());
				if (alias != null) {
					title = alias.getName();
					coordinates = alias.getCenter();
				} else {
					logger.warn("Invalid alias dbID: " + comment.getTableId());
				}
			}
		}
		result.setTitle(title);
		if (coordinates != null) {
			result.setxCoord(coordinates.getX());
			result.setyCoord(coordinates.getY());
			result.setZoom("" + (Configuration.MIN_ZOOM_LEVEL + model.getZoomLevels() - 1));
		}
		return result;
	}

	@Override
	public String createGson(CommentView object) {
		return new Gson().toJson(object);
	}

	@Override
	public Comment viewToObject(CommentView view) {
		throw new NotImplementedException();
	}
}
