package lcsb.mapviewer.services.view;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.faces.model.ListDataModel;

import lcsb.mapviewer.common.comparator.IntegerComparator;
import lcsb.mapviewer.model.Project;
import lcsb.mapviewer.model.map.layout.Layout;
import lcsb.mapviewer.model.user.PrivilegeType;
import lcsb.mapviewer.model.user.User;

import org.apache.log4j.Logger;
import org.primefaces.model.SelectableDataModel;

/**
 * This class represents view of the {@link User} that is available on the
 * client side.
 * 
 * @author Piotr Gawron
 * 
 */
public class UserView extends AbstractView<User> implements Comparable<UserView>, Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * Default class logger.
	 */
	private static Logger logger = Logger.getLogger(UserView.class);

	/**
	 * This data model is extension of faces {@link ListDataModel} and
	 * implements {@link SelectableDataModel} from primefaces library. It allows
	 * client to create a table (in xhtml) that allows to select element on it
	 * and call select event associated with the row. The model operates on
	 * {@link UserProjectPrivilegeView} that describe set of privileges to
	 * single project.
	 * 
	 * @author Piotr Gawron
	 * 
	 */
	public class ProjectDataModel extends ListDataModel<UserProjectPrivilegeView>
			implements SelectableDataModel<UserProjectPrivilegeView>, Serializable {

		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		/**
		 * Default constructor.
		 */
		public ProjectDataModel() {
		}

		/**
		 * Constructor that initialize the data.
		 * 
		 * @param data
		 *            data used in the model
		 */
		public ProjectDataModel(List<UserProjectPrivilegeView> data) {
			super(data);
		}

		@Override
		public Object getRowKey(UserProjectPrivilegeView object) {
			return object.toString();
		}

		@SuppressWarnings("unchecked")
		@Override
		public UserProjectPrivilegeView getRowData(String rowKey) {
			List<UserProjectPrivilegeView> privileges = (List<UserProjectPrivilegeView>) getWrappedData();

			for (UserProjectPrivilegeView privilege : privileges) {
				if ((privilege.toString()).equals(rowKey)) {
					return privilege;
				}
			}
			return null;
		}

	}

	/**
	 * Class representing prvileges to the specific project.
	 * 
	 * @author Piotr Gawron
	 * 
	 */
	public class UserProjectPrivilegeView extends AbstractView<Project> implements Serializable {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		/**
		 * {@link Project#projectId Project identifier}.
		 */
		private String projectId;

		/**
		 * List of privileges to the project.
		 */
		private List<PrivilegeView> projectPrivileges = new ArrayList<PrivilegeView>();

		/**
		 * List of layouts with privileges.
		 */
		private List<UserLayoutPrivilege> layoutPrivileges = new ArrayList<UserLayoutPrivilege>();

		/**
		 * Constructor that creates view for the project.
		 * 
		 * @param project
		 *            originasl project
		 */
		public UserProjectPrivilegeView(Project project) {
			super(project);
			this.projectId = project.getProjectId();
		}

		/**
		 * Default constructor.
		 */
		public UserProjectPrivilegeView() {
			super(null);
		}

		/**
		 * @return the projectId
		 * @see #projectId
		 */
		public String getProjectId() {
			return projectId;
		}

		/**
		 * @param projectId
		 *            the projectId to set
		 * @see #projectId
		 */
		public void setProjectId(String projectId) {
			this.projectId = projectId;
		}

		/**
		 * @return the projectPrivileges
		 * @see #projectPrivileges
		 */
		public List<PrivilegeView> getProjectPrivileges() {
			return projectPrivileges;
		}

		/**
		 * @param projectPrivileges
		 *            the projectPrivileges to set
		 * @see #projectPrivileges
		 */
		public void setProjectPrivileges(List<PrivilegeView> projectPrivileges) {
			this.projectPrivileges = projectPrivileges;
		}

		/**
		 * @return the layoutPrivileges
		 * @see #layoutPrivileges
		 */
		public List<UserLayoutPrivilege> getLayoutPrivileges() {
			return layoutPrivileges;
		}

		/**
		 * @param layoutPrivileges
		 *            the layoutPrivileges to set
		 * @see #layoutPrivileges
		 */
		public void setLayoutPrivileges(List<UserLayoutPrivilege> layoutPrivileges) {
			this.layoutPrivileges = layoutPrivileges;
		}

		/**
		 * Returns view of the {@link Privilege} for a given
		 * {@link PrivilegeType}.
		 * 
		 * @param pt
		 *            ttype of privilege
		 * @return view of the {@link Privilege} for a given
		 *         {@link PrivilegeType}
		 */
		public PrivilegeView getProjectPrivilegeByPrivilegeType(PrivilegeType pt) {
			for (PrivilegeView pv : projectPrivileges) {
				if (pv.getType() == pt) {
					return pv;
				}
			}
			logger.warn("Cannot find " + pt + " for projectID: " + this.getIdObject() + " (" + this.getProjectId()
					+ "). UserId: " + UserView.this.getIdObject() + " (" + UserView.this.getLogin() + ")");
			return null;
		}

	}

	/**
	 * Class representing prvileges to the specific
	 * {@link lcsb.mapviewer.db.model.map.layout.Layout Layout}.
	 * 
	 * @author Piotr Gawron
	 * 
	 */
	public class UserLayoutPrivilege extends AbstractView<Layout> {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		/**
		 * Name of the layout.
		 */
		private String layoutName;

		/**
		 * List of privileges for the layout.
		 */
		private List<PrivilegeView> layoutPrivileges = new ArrayList<PrivilegeView>();

		/**
		 * Default constructor.
		 * 
		 * @param layout
		 *            object for which the privilege set is created
		 */
		protected UserLayoutPrivilege(Layout layout) {
			super(layout);
		}

		/**
		 * Default constructor. Should be used only for deserialization.
		 */
		protected UserLayoutPrivilege() {
		}

		/**
		 * @return the layoutName
		 * @see #layoutName
		 */
		public String getLayoutName() {
			return layoutName;
		}

		/**
		 * @param layoutName
		 *            the layoutName to set
		 * @see #layoutName
		 */
		public void setLayoutName(String layoutName) {
			this.layoutName = layoutName;
		}

		/**
		 * @return the layoutPrivileges
		 * @see #layoutPrivileges
		 */
		public List<PrivilegeView> getLayoutPrivileges() {
			return layoutPrivileges;
		}

		/**
		 * @param layoutPrivileges
		 *            the layoutPrivileges to set
		 * @see #layoutPrivileges
		 */
		public void setLayoutPrivileges(List<PrivilegeView> layoutPrivileges) {
			this.layoutPrivileges = layoutPrivileges;
		}

	}

	/**
	 * Use login.
	 */
	private String login;

	/**
	 * User password (can be set only by the client side, when view is created
	 * password should be set to empty, simple precaution not to reveal
	 * passwords).
	 */
	private String password;
	
	/**
	 * User password 2.
	 */
	private String password2;

	/**
	 * Old password.
	 */
	private String oldPassword;

	/**
	 * Old Encrypted password.
	 */
	private String cryptedPassword;

	/**
	 * Name of the user.
	 */
	private String name;

	/**
	 * Family name of the user.
	 */
	private String surname;

	/**
	 * User email address.
	 */
	private String email;

	/**
	 * List of general privielges.
	 */
	private List<PrivilegeView> basicPrivileges = new ArrayList<PrivilegeView>();
	/**
	 * List of privileges divided by the projects.
	 */

	private List<UserProjectPrivilegeView> projectPrivileges = new ArrayList<UserProjectPrivilegeView>();
	/**
	 * Data model used for managing project privileges.
	 */
	private ProjectDataModel pdm = null;

	/**
	 * Constructor that create user view from the user data.
	 * 
	 * @param user
	 *            orignal user
	 */
	protected UserView(User user) {
		super(user);
		this.login = user.getLogin();
		this.password = "";
		this.password2 = "";
		this.oldPassword = "";
		this.cryptedPassword = user.getCryptedPassword();
		this.name = user.getName();
		this.surname = user.getSurname();
		this.email = user.getEmail();
	}

	/**
	 * Default constructor.
	 */
	protected UserView() {
		super(null);
	}

	/**
	 * Returns object with privileges for the given project.
	 * 
	 * @param projectId
	 *            identrifier of the project
	 * @return object with privileges for the given project
	 */
	public UserProjectPrivilegeView getProjectPrivilegeByProjectId(int projectId) {
		for (UserProjectPrivilegeView projectPrivilege : getProjectPrivileges()) {
			if (projectPrivilege.getIdObject() == projectId) {
				return projectPrivilege;
			}
		}
		if (projectId != 0) {
			logger.warn("Cannot find project privileges for projectID: " + projectId + ". User: " + getIdObject() + " ("
					+ getLogin() + ").");
		}
		return null;
	}

	/**
	 * 
	 * @return {@link #pdm}
	 */
	public ProjectDataModel getPdm() {
		if (pdm == null) {
			pdm = new ProjectDataModel(projectPrivileges);
		}
		return pdm;
	}

	/**
	 * 
	 * @param pdm
	 *            new {@link #pdm} object
	 */
	public void setPdm(ProjectDataModel pdm) {
		this.pdm = pdm;
	}

	/**
	 * Return object with privileges for layout given in the parameter.
	 * 
	 * @param layoutId
	 *            identifier of the layout
	 * @return object with privileges for layout
	 */
	public UserLayoutPrivilege getLayoutPrivilegeByLayoutId(int layoutId) {
		for (UserProjectPrivilegeView projectPrivilege : getProjectPrivileges()) {
			for (UserLayoutPrivilege layoutPrivilege : projectPrivilege.getLayoutPrivileges()) {
				if (layoutPrivilege.getIdObject() == layoutId) {
					return layoutPrivilege;
				}
			}
		}
		logger.warn("Cannot find project privileges for layoutID: " + layoutId);
		return null;
	}

	@Override
	public int compareTo(UserView o) {
		IntegerComparator comparator = new IntegerComparator();
		if (o == null) {
			comparator.compare(getIdObject(), null);
		}
		return comparator.compare(getIdObject(), o.getIdObject());

	}

	/**
	 * @return the login
	 * @see #login
	 */
	public String getLogin() {
		return login;
	}

	/**
	 * @param login
	 *            the login to set
	 * @see #login
	 */
	public void setLogin(String login) {
		this.login = login;
	}

	/**
	 * @return the password
	 * @see #password
	 */
	public String getPassword() {
		return password;
	}

	/**
	 * @param password
	 *            the password to set
	 * @see #password
	 */
	public void setPassword(String password) {
		this.password = password;
	}

	/**
	 * @return the name
	 * @see #name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name
	 *            the name to set
	 * @see #name
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @return the surname
	 * @see #surname
	 */
	public String getSurname() {
		return surname;
	}

	/**
	 * @param surname
	 *            the surname to set
	 * @see #surname
	 */
	public void setSurname(String surname) {
		this.surname = surname;
	}

	/**
	 * @return the email
	 * @see #email
	 */
	public String getEmail() {
		return email;
	}

	/**
	 * @param email
	 *            the email to set
	 * @see #email
	 */
	public void setEmail(String email) {
		this.email = email;
	}

	/**
	 * @return the basicPrivileges
	 * @see #basicPrivileges
	 */
	public List<PrivilegeView> getBasicPrivileges() {
		return basicPrivileges;
	}

	/**
	 * @param basicPrivileges
	 *            the basicPrivileges to set
	 * @see #basicPrivileges
	 */
	public void setBasicPrivileges(List<PrivilegeView> basicPrivileges) {
		this.basicPrivileges = basicPrivileges;
	}

	/**
	 * @return the projectPrivileges
	 * @see #projectPrivileges
	 */
	public List<UserProjectPrivilegeView> getProjectPrivileges() {
		return projectPrivileges;
	}

	/**
	 * @param projectPrivileges
	 *            the projectPrivileges to set
	 * @see #projectPrivileges
	 */
	public void setProjectPrivileges(List<UserProjectPrivilegeView> projectPrivileges) {
		this.projectPrivileges = projectPrivileges;
	}

	/**
	 * @return old password as typed by the user.
	 */
	public String getOldPassword() {
		return oldPassword;
	}

	/**
	 * @param oldPassword the field for the user to put in the old password,
	 */
	public void setOldPassword(String oldPassword) {
		this.oldPassword = oldPassword;
	}

	/**
	 * @return password crypted and stored in DB.
	 */
	public String getCryptedPassword() {
		return cryptedPassword;
	}

	/**
	 * @param cryptedPassword crypted password stored in DB
	 */
	public void setCryptedPassword(String cryptedPassword) {
		this.cryptedPassword = cryptedPassword;
	}

	/**
	 * @return password2.
	 */
	public String getPassword2() {
		return password2;
	}

	/**
	 * @param password2 to verify that the password match.
	 */
	public void setPassword2(String password2) {
		this.password2 = password2;
	}
}
