package lcsb.mapviewer.services.view;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import lcsb.mapviewer.model.Project;
import lcsb.mapviewer.model.map.layout.Layout;
import lcsb.mapviewer.model.user.BasicPrivilege;
import lcsb.mapviewer.model.user.ObjectPrivilege;
import lcsb.mapviewer.model.user.PrivilegeType;
import lcsb.mapviewer.model.user.User;
import lcsb.mapviewer.persist.dao.ProjectDao;
import lcsb.mapviewer.services.utils.InvalidPrivilegeException;
import lcsb.mapviewer.services.view.UserView.UserLayoutPrivilege;
import lcsb.mapviewer.services.view.UserView.UserProjectPrivilegeView;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;

import lcsb.mapviewer.common.exception.NotImplementedException;

import com.google.gson.Gson;

/**
 * Factory class for {@link UserView} class.
 * 
 * @author Piotr Gawron
 * 
 */
public class UserViewFactory extends AbstractViewFactory<User, UserView> {
	
	/**
	 * Default class logger.
	 */
	@SuppressWarnings("unused")
	private static Logger				 logger	= Logger.getLogger(UserViewFactory.class);

	/**
	 * Data access object for projects.
	 */
	@Autowired
	private ProjectDao					 projectDao;
	
	/**
	 * Factory object for {@link PrivilegeView} elements.
	 */
	@Autowired
	private PrivilegeViewFactory privilegeViewFactory;

	@Override
	public UserView create(User user) {
		return create(user, projectDao.getAll());
	}

	/**
	 * Creates {@link UserView} obejct for given user and given list of projects.
	 * 
	 * @param user
	 *          object for which {@link UserView} element will be created
	 * @param projects
	 *          list of project for which privileges will be created as
	 *          {@link PrivilegeView} elements in the result
	 * @return {@link UserView} obejct for given user
	 */
	public UserView create(User user, List<Project> projects) {
		UserView result = null;
		if (user == null) {
			result = new UserView();
			for (Project project : projects) {
				result.getProjectPrivileges().add(result.new UserProjectPrivilegeView(project));
			}
			for (PrivilegeType type : getBasicTypes()) {
				result.getBasicPrivileges().add(privilegeViewFactory.create(type));
			}
			for (UserProjectPrivilegeView projectPrivilege : result.getProjectPrivileges()) {
				for (PrivilegeType type : getObjectTypes()) {
					if (type.getPrivilegeObjectType().equals(Project.class)) {
						projectPrivilege.getProjectPrivileges().add(privilegeViewFactory.create(type));
					}
				}
			}
		} else {
			result = new UserView(user);

			for (Project project : projects) {
				result.getProjectPrivileges().add(result.new UserProjectPrivilegeView(project));
			}

			setPrivilegeStatusFromDb(user, result);

			createUnknownPrivileges(result);
		}

		Collections.sort(result.getBasicPrivileges());
		for (UserProjectPrivilegeView row1 : result.getProjectPrivileges()) {
			Collections.sort(row1.getProjectPrivileges());
		}

		return result;
	}

	/**
	 * Creates privileges for user that doesn't exists in database.
	 * 
	 * @param row
	 *          user for which we want to add missing privileges
	 */
	private void createUnknownPrivileges(UserView row) {
		Set<PrivilegeType> knownPrivileges = new HashSet<PrivilegeType>();
		for (PrivilegeView privilege : row.getBasicPrivileges()) {
			knownPrivileges.add(privilege.getType());
		}

		for (PrivilegeType type : getBasicTypes()) {
			if (!knownPrivileges.contains(type)) {
				row.getBasicPrivileges().add(privilegeViewFactory.create(type));
			}
		}
		for (UserProjectPrivilegeView projectPrivilege : row.getProjectPrivileges()) {
			knownPrivileges.clear();
			for (PrivilegeView privilege : projectPrivilege.getProjectPrivileges()) {
				knownPrivileges.add(privilege.getType());
			}
			for (PrivilegeType type : getObjectTypes()) {
				if (type.getPrivilegeObjectType().equals(Project.class)) {
					if (!knownPrivileges.contains(type)) {
						projectPrivilege.getProjectPrivileges().add(privilegeViewFactory.create(type));
					}
				}
			}
		}
	}

	/**
	 * Method that assignes privileges from original user object into view
	 * representation userView.
	 * 
	 * @param user
	 *          original object retrieved from database
	 * @param userView
	 *          representation of the user passed later on to higher layer
	 */
	private void setPrivilegeStatusFromDb(User user, UserView userView) {
		for (BasicPrivilege privilege : user.getPrivileges()) {
			if (privilege.getClass().equals(BasicPrivilege.class)) {
				userView.getBasicPrivileges().add(privilegeViewFactory.create(privilege));
			} else if (privilege.getClass().equals(ObjectPrivilege.class)) {
				if (privilege.getType().getPrivilegeObjectType().equals(Project.class)) {
					int projectId = ((ObjectPrivilege) privilege).getIdObject();
					UserProjectPrivilegeView projectPrivilege = userView.getProjectPrivilegeByProjectId(projectId);
					if (projectPrivilege != null) {
						projectPrivilege.getProjectPrivileges().add(privilegeViewFactory.create(privilege));
					}
				} else if (privilege.getType().getPrivilegeObjectType().equals(Layout.class)) {
					int layoutId = ((ObjectPrivilege) privilege).getIdObject();
					UserLayoutPrivilege layoutPrivilege = userView.getLayoutPrivilegeByLayoutId(layoutId);
					if (layoutPrivilege != null) {
						layoutPrivilege.getLayoutPrivileges().add(privilegeViewFactory.create(privilege));
					}
				} else {
					throw new InvalidPrivilegeException("Unknown class for object privilege: " + privilege.getType().getPrivilegeObjectType());
				}
			} else {
				throw new InvalidPrivilegeException("Unknown privilege type: " + privilege.getType());
			}
		}
	}

	/**
	 * Returns list of basic privilege types (privileges that doesn't refer to
	 * another object).
	 * 
	 * @return list of basic privilege types
	 */
	public List<PrivilegeType> getBasicTypes() {
		if (basicTypes == null) {
			fillTypeList();
		}
		return basicTypes;
	}

	/**
	 * List of type privileges that refers to {@link BasicPrivilege} class.
	 */
	private List<PrivilegeType>	basicTypes	= null;
	/**
	 * List of type privileges that refers to {@link ObjectPrivilege} class.
	 */
	private List<PrivilegeType>	objectTypes	= null;

	/**
	 * Returns list of object privilege types (privileges that refer to another
	 * object).
	 * 
	 * @return list of object privilege types
	 */
	public List<PrivilegeType> getObjectTypes() {
		if (objectTypes == null) {
			fillTypeList();
		}
		return objectTypes;
	}

	/**
	 * This method initializes {@link #basicTypes} and {@link #objectTypes} lists.
	 */
	private void fillTypeList() {
		basicTypes = new ArrayList<PrivilegeType>();
		objectTypes = new ArrayList<PrivilegeType>();

		PrivilegeType[] types = PrivilegeType.values();

		for (PrivilegeType privilegeType : types) {
			if (privilegeType.getPrivilegeClassType().equals(BasicPrivilege.class)) {
				basicTypes.add(privilegeType);
			} else if (privilegeType.getPrivilegeClassType().equals(ObjectPrivilege.class)) {
				objectTypes.add(privilegeType);
			} else {
				throw new InvalidPrivilegeException("Unknown privilege type: " + privilegeType.getPrivilegeClassType());
			}
		}

	}

	@Override
	public String createGson(UserView object) {
		return new Gson().toJson(object);
	}

	@Override
	public User viewToObject(UserView view) {
		throw new NotImplementedException();
	}

}
